"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parser_combinators_1 = require("@reactive-js/parser-combinators");
const httpGrammar_1 = require("./httpGrammar");
const option_1 = require("@reactive-js/option");
const httpHeaders_1 = require("./httpHeaders");
const pipe_1 = require("@reactive-js/pipe");
exports.mustRevalidate = {
    directive: "must-revalidate",
    value: "",
};
exports.noStore = { directive: "no-store", value: "" };
exports.noTransform = {
    directive: "no-transform",
    value: "",
};
exports.onlyIfCached = {
    directive: "only-if-cached",
    value: "",
};
exports.public_ = { directive: "public", value: "" };
exports.proxyRevalidate = {
    directive: "proxy-revalidate",
    value: "",
};
exports.maxAge = (value) => ({
    directive: "max-age",
    value: value.toFixed(0),
});
exports.maxStale = (value) => ({
    directive: "max-stale",
    value: value.toFixed(0),
});
exports.minFresh = (value) => ({
    directive: "min-fresh",
    value: value.toFixed(0),
});
exports.sharedMaxAge = (value) => ({
    directive: "s-maxage",
    value: value.toFixed(0),
});
exports.noCache = (headers = []) => ({
    directive: "no-cache",
    value: headers.join(","),
});
exports.private_ = (headers = []) => ({
    directive: "private",
    value: headers.join(","),
});
const pOptionalEquals = parser_combinators_1.optional(parser_combinators_1.pEquals);
const pCacheDirective = (charStream) => {
    const directive = httpGrammar_1.pToken(charStream);
    const hasValue = option_1.isSome(pOptionalEquals(charStream));
    const value = hasValue ? httpGrammar_1.pTokenOrQuotedString(charStream) : "";
    return { directive, value };
};
exports.parseCacheDirective = parser_combinators_1.parseWith(pCacheDirective);
exports.parseCacheDirectiveOrThrow = parser_combinators_1.parseWithOrThrow(pCacheDirective);
exports.cacheDirectiveToString = ({ directive, value, }) => value.length > 0
    ? `${directive}=${httpGrammar_1.toTokenOrQuotedString(value)}`
    : `${directive}`;
const parseCacheDirectiveList = pipe_1.pipe(pCacheDirective, httpGrammar_1.httpList, parser_combinators_1.parseWith);
exports.parseCacheControlFromHeaders = (headers) => {
    var _a;
    const cacheControl = httpHeaders_1.getHeaderValue(headers, "Cache-Control");
    return option_1.isSome(cacheControl)
        ? (_a = parseCacheDirectiveList(cacheControl)) !== null && _a !== void 0 ? _a : [] : [];
};
exports.writeHttpCacheControlHeader = (cacheControl, writeHeader) => {
    if (cacheControl.length > 0) {
        writeHeader("Cache-Control", cacheControl.map(exports.cacheDirectiveToString).join(","));
    }
};
//# sourceMappingURL=cacheDirective.js.map