"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const option_1 = require("@reactive-js/option");
const httpContent_1 = require("./httpContent");
const httpDateTime_1 = require("./httpDateTime");
const entityTag_1 = require("./entityTag");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
const cacheDirective_1 = require("./cacheDirective");
exports.createHttpResponse = (statusCode, _a = {}) => {
    var { cacheControl, content, etag, expires, headers, lastModified, location, preferences, vary } = _a, rest = tslib_1.__rest(_a, ["cacheControl", "content", "etag", "expires", "headers", "lastModified", "location", "preferences", "vary"]);
    return (Object.assign(Object.assign({}, rest), { cacheControl: (cacheControl !== null && cacheControl !== void 0 ? cacheControl : []).map(cc => typeof cc === "string" ? cacheDirective_1.parseCacheDirectiveOrThrow(cc) : cc), content, etag: typeof etag === "string" ? entityTag_1.parseETagOrThrow(etag) : etag, expires, headers: httpHeaders_1.filterHeaders(headers !== null && headers !== void 0 ? headers : {}), lastModified: typeof lastModified === "string"
            ? httpDateTime_1.parseHttpDateTime(lastModified)
            : lastModified instanceof Date
                ? lastModified.getTime()
                : lastModified, location: typeof location === "string" ? new URL(location) : location, preferences: option_1.isSome(preferences)
            ? httpPreferences_1.createHttpPreferences(preferences)
            : undefined, statusCode, vary: vary !== null && vary !== void 0 ? vary : [] }));
};
exports.parseHttpResponseFromHeaders = (statusCode, headers, body) => {
    var _a, _b, _c;
    const cacheControl = cacheDirective_1.parseCacheControlFromHeaders(headers);
    const content = httpContent_1.parseHttpContentFromHeaders(headers, body);
    const etag = entityTag_1.parseETag((_a = httpHeaders_1.getHeaderValue(headers, "ETag")) !== null && _a !== void 0 ? _a : "");
    const expires = httpDateTime_1.parseHttpDateTime((_b = httpHeaders_1.getHeaderValue(headers, "Expires")) !== null && _b !== void 0 ? _b : "");
    const lastModified = httpDateTime_1.parseHttpDateTime((_c = httpHeaders_1.getHeaderValue(headers, "Last-Modified")) !== null && _c !== void 0 ? _c : "");
    const locationHeader = headers.location;
    const location = option_1.isSome(locationHeader) ? new URL(locationHeader) : option_1.none;
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const vary = [];
    return {
        cacheControl,
        etag,
        expires,
        lastModified,
        content,
        headers,
        location,
        preferences,
        statusCode,
        vary,
    };
};
const writeCoreHttpResponseHeaders = ({ cacheControl, etag, expires, headers, lastModified, location, preferences, vary, }, writeHeader) => {
    cacheDirective_1.writeHttpCacheControlHeader(cacheControl, writeHeader);
    if (option_1.isSome(etag)) {
        writeHeader("ETag", entityTag_1.entityTagToString(etag));
    }
    if (option_1.isSome(expires)) {
        writeHeader("Expires", httpDateTime_1.httpDateTimeToString(expires));
    }
    if (option_1.isSome(lastModified)) {
        writeHeader("Last-Modified", httpDateTime_1.httpDateTimeToString(lastModified));
    }
    if (option_1.isSome(location)) {
        writeHeader("Location", location.toString());
    }
    if (option_1.isSome(preferences)) {
        httpPreferences_1.writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    if (vary.length > 0) {
        writeHeader("Vary", vary.join(","));
    }
    httpHeaders_1.writeHttpHeaders(headers, writeHeader);
};
exports.writeHttpResponseHeaders = (response, writeHeader) => {
    const { content } = response;
    writeCoreHttpResponseHeaders(response, writeHeader);
    if (option_1.isSome(content)) {
        httpContent_1.writeHttpContentHeaders(content, writeHeader);
    }
};
exports.checkIfNotModified = ({ cacheControl, method, preconditions, }) => response => {
    var _a;
    const { etag, lastModified } = response;
    const { statusCode, content: _ } = response, responseWithoutContent = tslib_1.__rest(response, ["statusCode", "content"]);
    const methodSupportsConditionalResponse = method === "GET" || method === "HEAD";
    const statusCodeSupportsConditionalResponse = statusCode >= 200 && statusCode < 300;
    const isNoCacheRequest = cacheControl.findIndex(({ directive }) => directive === "no-cache") >= 0;
    const etagMatch = option_1.isSome(etag) &&
        ((preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) === "*" ||
            ((preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) || []).findIndex(({ tag }) => tag === etag.tag) >= 0);
    const notModifiedSince = (lastModified !== null && lastModified !== void 0 ? lastModified : Number.MAX_SAFE_INTEGER) <=
        ((_a = preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince) !== null && _a !== void 0 ? _a : Number.MIN_SAFE_INTEGER);
    const match = option_1.isSome(etag) &&
        option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) &&
        option_1.isSome(lastModified) &&
        option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince)
        ? notModifiedSince && etagMatch
        : option_1.isSome(etag) && option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch)
            ? etagMatch
            : notModifiedSince;
    return methodSupportsConditionalResponse &&
        statusCodeSupportsConditionalResponse &&
        !isNoCacheRequest &&
        match
        ? Object.assign(Object.assign({}, responseWithoutContent), { statusCode: 304 }) : response;
};
//# sourceMappingURL=httpResponse.js.map