"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parser_combinators_1 = require("@reactive-js/parser-combinators");
const httpGrammar_1 = require("./httpGrammar");
exports.pMediaType = charStream => {
    const type = httpGrammar_1.pToken(charStream);
    parser_combinators_1.pForwardSlash(charStream);
    const subtype = httpGrammar_1.pToken(charStream);
    const params = httpGrammar_1.pParams(charStream);
    return {
        type,
        subtype,
        params,
    };
};
exports.parseMediaType = parser_combinators_1.parseWith(exports.pMediaType);
exports.parseMediaTypeOrThrow = parser_combinators_1.parseWithOrThrow(exports.pMediaType);
exports.mediaTypeToString = ({ type, subtype, params, }) => {
    const stringParams = Object.entries(params)
        .map(([k, v]) => `${k}=${httpGrammar_1.toTokenOrQuotedString(v)}`)
        .join("; ");
    return `${type}/${subtype}${stringParams.length > 0 ? ";" + stringParams : ""}`;
};
const compressionBlacklist = [
    "text/event-stream",
];
const textSubtypes = ["html", "json", "text", "xml"];
exports.mediaTypeIsCompressible = (db) => ({ type, subtype }) => {
    var _a, _b;
    const mediaType = exports.mediaTypeToString({ type, subtype, params: {} });
    const blackListed = compressionBlacklist.includes(mediaType);
    const compressible = (_b = (_a = db[mediaType]) === null || _a === void 0 ? void 0 : _a.compressible) !== null && _b !== void 0 ? _b : false;
    const typeIsText = type === "text";
    const subtypeIsText = textSubtypes.filter(x => subtype.endsWith(x)).length > 0;
    return !blackListed && (compressible || typeIsText || subtypeIsText);
};
//# sourceMappingURL=mediaType.js.map