import { optional, pEquals, parseWith, parseWithOrThrow, } from "@reactive-js/parser-combinators";
import { pToken, pTokenOrQuotedString, toTokenOrQuotedString, httpList, } from "./httpGrammar";
import { isSome } from "@reactive-js/option";
import { getHeaderValue } from "./httpHeaders";
import { pipe } from "@reactive-js/pipe";
export const mustRevalidate = {
    directive: "must-revalidate",
    value: "",
};
export const noStore = { directive: "no-store", value: "" };
export const noTransform = {
    directive: "no-transform",
    value: "",
};
export const onlyIfCached = {
    directive: "only-if-cached",
    value: "",
};
export const public_ = { directive: "public", value: "" };
export const proxyRevalidate = {
    directive: "proxy-revalidate",
    value: "",
};
export const maxAge = (value) => ({
    directive: "max-age",
    value: value.toFixed(0),
});
export const maxStale = (value) => ({
    directive: "max-stale",
    value: value.toFixed(0),
});
export const minFresh = (value) => ({
    directive: "min-fresh",
    value: value.toFixed(0),
});
export const sharedMaxAge = (value) => ({
    directive: "s-maxage",
    value: value.toFixed(0),
});
export const noCache = (headers = []) => ({
    directive: "no-cache",
    value: headers.join(","),
});
export const private_ = (headers = []) => ({
    directive: "private",
    value: headers.join(","),
});
const pOptionalEquals = optional(pEquals);
const pCacheDirective = (charStream) => {
    const directive = pToken(charStream);
    const hasValue = isSome(pOptionalEquals(charStream));
    const value = hasValue ? pTokenOrQuotedString(charStream) : "";
    return { directive, value };
};
export const parseCacheDirective = parseWith(pCacheDirective);
export const parseCacheDirectiveOrThrow = parseWithOrThrow(pCacheDirective);
export const cacheDirectiveToString = ({ directive, value, }) => value.length > 0
    ? `${directive}=${toTokenOrQuotedString(value)}`
    : `${directive}`;
const parseCacheDirectiveList = pipe(pCacheDirective, httpList, parseWith);
export const parseCacheControlFromHeaders = (headers) => {
    var _a;
    const cacheControl = getHeaderValue(headers, "Cache-Control");
    return isSome(cacheControl)
        ? (_a = parseCacheDirectiveList(cacheControl)) !== null && _a !== void 0 ? _a : [] : [];
};
export const writeHttpCacheControlHeader = (cacheControl, writeHeader) => {
    if (cacheControl.length > 0) {
        writeHeader("Cache-Control", cacheControl.map(cacheDirectiveToString).join(","));
    }
};
//# sourceMappingURL=cacheDirective.js.map