import { __rest } from "tslib";
import { isSome, none } from "@reactive-js/option";
import { writeHttpContentHeaders, parseHttpContentFromHeaders, } from "./httpContent";
import { parseHttpDateTime, httpDateTimeToString } from "./httpDateTime";
import { entityTagToString, parseETag, parseETagOrThrow } from "./entityTag";
import { writeHttpHeaders, getHeaderValue, filterHeaders, } from "./httpHeaders";
import { parseHttpPreferencesFromHeaders, writeHttpPreferenceHeaders, createHttpPreferences, } from "./httpPreferences";
import { parseCacheControlFromHeaders, writeHttpCacheControlHeader, parseCacheDirectiveOrThrow, } from "./cacheDirective";
export const createHttpResponse = (statusCode, _a = {}) => {
    var { cacheControl, content, etag, expires, headers, lastModified, location, preferences, vary } = _a, rest = __rest(_a, ["cacheControl", "content", "etag", "expires", "headers", "lastModified", "location", "preferences", "vary"]);
    return (Object.assign(Object.assign({}, rest), { cacheControl: (cacheControl !== null && cacheControl !== void 0 ? cacheControl : []).map(cc => typeof cc === "string" ? parseCacheDirectiveOrThrow(cc) : cc), content, etag: typeof etag === "string" ? parseETagOrThrow(etag) : etag, expires, headers: filterHeaders(headers !== null && headers !== void 0 ? headers : {}), lastModified: typeof lastModified === "string"
            ? parseHttpDateTime(lastModified)
            : lastModified instanceof Date
                ? lastModified.getTime()
                : lastModified, location: typeof location === "string" ? new URL(location) : location, preferences: isSome(preferences)
            ? createHttpPreferences(preferences)
            : undefined, statusCode, vary: vary !== null && vary !== void 0 ? vary : [] }));
};
export const parseHttpResponseFromHeaders = (statusCode, headers, body) => {
    var _a, _b, _c;
    const cacheControl = parseCacheControlFromHeaders(headers);
    const content = parseHttpContentFromHeaders(headers, body);
    const etag = parseETag((_a = getHeaderValue(headers, "ETag")) !== null && _a !== void 0 ? _a : "");
    const expires = parseHttpDateTime((_b = getHeaderValue(headers, "Expires")) !== null && _b !== void 0 ? _b : "");
    const lastModified = parseHttpDateTime((_c = getHeaderValue(headers, "Last-Modified")) !== null && _c !== void 0 ? _c : "");
    const locationHeader = headers.location;
    const location = isSome(locationHeader) ? new URL(locationHeader) : none;
    const preferences = parseHttpPreferencesFromHeaders(headers);
    const vary = [];
    return {
        cacheControl,
        etag,
        expires,
        lastModified,
        content,
        headers,
        location,
        preferences,
        statusCode,
        vary,
    };
};
const writeCoreHttpResponseHeaders = ({ cacheControl, etag, expires, headers, lastModified, location, preferences, vary, }, writeHeader) => {
    writeHttpCacheControlHeader(cacheControl, writeHeader);
    if (isSome(etag)) {
        writeHeader("ETag", entityTagToString(etag));
    }
    if (isSome(expires)) {
        writeHeader("Expires", httpDateTimeToString(expires));
    }
    if (isSome(lastModified)) {
        writeHeader("Last-Modified", httpDateTimeToString(lastModified));
    }
    if (isSome(location)) {
        writeHeader("Location", location.toString());
    }
    if (isSome(preferences)) {
        writeHttpPreferenceHeaders(preferences, writeHeader);
    }
    if (vary.length > 0) {
        writeHeader("Vary", vary.join(","));
    }
    writeHttpHeaders(headers, writeHeader);
};
export const writeHttpResponseHeaders = (response, writeHeader) => {
    const { content } = response;
    writeCoreHttpResponseHeaders(response, writeHeader);
    if (isSome(content)) {
        writeHttpContentHeaders(content, writeHeader);
    }
};
export const checkIfNotModified = ({ cacheControl, method, preconditions, }) => response => {
    var _a;
    const { etag, lastModified } = response;
    const { statusCode, content: _ } = response, responseWithoutContent = __rest(response, ["statusCode", "content"]);
    const methodSupportsConditionalResponse = method === "GET" || method === "HEAD";
    const statusCodeSupportsConditionalResponse = statusCode >= 200 && statusCode < 300;
    const isNoCacheRequest = cacheControl.findIndex(({ directive }) => directive === "no-cache") >= 0;
    const etagMatch = isSome(etag) &&
        ((preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) === "*" ||
            ((preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) || []).findIndex(({ tag }) => tag === etag.tag) >= 0);
    const notModifiedSince = (lastModified !== null && lastModified !== void 0 ? lastModified : Number.MAX_SAFE_INTEGER) <=
        ((_a = preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince) !== null && _a !== void 0 ? _a : Number.MIN_SAFE_INTEGER);
    const match = isSome(etag) &&
        isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) &&
        isSome(lastModified) &&
        isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince)
        ? notModifiedSince && etagMatch
        : isSome(etag) && isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch)
            ? etagMatch
            : notModifiedSince;
    return methodSupportsConditionalResponse &&
        statusCodeSupportsConditionalResponse &&
        !isNoCacheRequest &&
        match
        ? Object.assign(Object.assign({}, responseWithoutContent), { statusCode: 304 }) : response;
};
//# sourceMappingURL=httpResponse.js.map